
import java.lang.Integer;

/**
 * Determines whether a number provided on the command line is a prime
 * number. The means used to determine this is a combination of the Sieve
 * of Eratosthenes and trial division. The Sieve is used to build up the
 * set of primes from 2 to (n^(1/2)), those primes are then used in an
 * attempt to factor the candidate prime.
 *
 * @author Christian Ratliff
 */
public class SerialPrime
{
  /** A knockout array used to mark prime numbers. This value
   * is reused over and over to speed primality analysis. */
  private boolean[] primeArray;

  /**
   * Prepares this class by frontloading the prime array with
   * the initial run of simple primes (0, 1, 2, 3). The numbers
   * 0 and 1 are neither prime nor composite.
   */
  public SerialPrime() {
    this.primeArray = new boolean[4];

    for (int x = 0; x < 4; x++) { this.primeArray[x] = true; }
  }

  /**
   * Determines if the number provided is prime by using either
   * a simple lookup test or trial division.
   *
   * @return <code>true</code> if the number is prime.
   */
  public boolean checkPrimeness(int n) {
    /* If the number is already within the region tested, then there
     * is no need to even try and factor it. */
    if (n < this.primeArray.length) {
      return this.primeArray[n];
    }

    /* Size the array to square root of 'n'. */
    int len = (int)Math.sqrt((double)n);

    if (len > this.primeArray.length) {
      boolean[] newArray = new boolean[len];

      /* copy in the values in the old array. */
      for (int x = 0; x < this.primeArray.length; x++) {
	newArray[x] = this.primeArray[x];
      }

      /* run the sieve against these new values. */
      for (int x = this.primeArray.length; x < len; x++) {
	newArray[x] = true;

	/* Divide this value by all the preceeding primes. */
	for (int y = 2; y < x; y++) {
	  /* If this value (y) is prime, the mod (x) by it and
	   * see if there _is_ a result. If not, composite. */
	  if (newArray[y] == true && (x % y) == 0) {
	    newArray[x] = false;
	    break;
	  }
	}
      }
      this.primeArray = newArray;
    }

    /* Now try and divide the value by all the primes in the array. */
    for (int x = 2; x < this.primeArray.length; x++) {
      if (this.primeArray[x] == true && (n % x) == 0) {
	return false;
      }
    }
    return true;
  }

  /**
   * Tests the SerialPrime class.
   *
   * @param argv The string arguments provided on the command line.
   */
  public static void main(String[] argv) {
    SerialPrime ptest = new SerialPrime();

    if (argv.length == 0) {
      System.out.println("Usage: java SerialPrime n [n ...]");
      return;
    }

    for (int x = 0; x < argv.length; x++) {
      if (ptest.checkPrimeness(Integer.parseInt(argv[x]))) {
	System.out.println("Number ["+argv[x]+"] IS prime.");
      } else {
	System.out.println("Number ["+argv[x]+"] is NOT prime.");
      }
    }
  }
}

    /*
    System.out.println("PRIMES:");
    for (int x = 2; x < ptest.primeArray.length; x++) {
      if (ptest.primeArray[x] == true) {
	System.out.print(" "+x+",");
      }
    }
    System.out.println(".");
    */
