
import java.util.TreeSet;

/**
 * Stores a precomputed set of prime numbers for later lookup.
 *
 * @author Christian Ratliff
 */
public class PrimeCache {
  /** An unsynchronized storage site for incoming prime numbers. */
  private TreeSet primes;
  /** Indicates the largest prime number stored in the set. */
  private Integer maxPrime;

  /**
   * Initializes an instance of the PrimeCache.
   */
  public PrimeCache() {
    super();
    primes = new TreeSet();
    maxPrime = new Integer(0);
  }

  /**
   * Places the provided prime into the cache and increases the
   * largest prime number indicator.
   *
   * @param newPrime The new prime number to add, it must be larger
   *       than the current biggest prime number.
   *
   * @throws IllegalArgumentException If the provided prime number
   *        is smaller than the current largest.
   */
  public void addPrimeToCache(Integer newPrime) {
    if (newPrime.compareTo(this.maxPrime) <= 0) {
      throw new IllegalArgumentException("The new prime number is less than or equal to the current max prime.");
    }

    this.maxPrime = newPrime;
    this.primes.add(newPrime);
  }
  
  /**
   *  Test the primeness of a number by looking it up in the cache.
   *
   * Wizards: what is the interesting corner case here?
   *
   * @return true if the value is prime, false otherwise.
   */
  public boolean isPrime(Integer testPrime) {
    /* Cycle in a busywait until this number falls within the
     * range of the prime cache. */
    while (testPrime.compareTo(this.maxPrime) > 0) {
      Thread.yield();
    }

    return this.primes.contains(testPrime);
  }
}

/* add Thread.yield(); to addPrimeToCache */
/* change the methods to be synchronized. */
/* move to the Blocked state: try { Thread.sleep(50); } catch (InterruptedException ie) { } */

