import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;

/**
 * Searches a given range for prime numbers, adding each prime
 * it finds to a prime numbers cache. The sieve algorithm
 * works by establishing an array from 1 to n
 *
 * @author Christian Ratliff
 */
public class SieveThread extends Thread {
  /** The knockout array for prime numbers from 1 to n */
  boolean[] primeArray;
  /** The cache where found prime numbers will be stored. */
  PrimeCache cache;

  /**
   * Initializes the sieve instance by allocating the prime
   * array to the size indicated by the max prime parameter.
   *
   * @param maxPrime The upper bound on the range.
   * @param cache The class which caches the found prime
   *       numbers.
   */
  public SieveThread(int maxPrime, PrimeCache cache) {
    this.primeArray = new boolean[maxPrime];
    this.cache = cache;
  }

  /**
   * Performs the actual work of the thread by filtering
   * the array of composites. Any prime numbers it finds
   * are added to the prime number cache.
   */
  public void run() {
    /* Fill in the array as if every number was prime. */
    for (int n = 0; n < this.primeArray.length; n++) {
      this.primeArray[n] = true;
    }

    /* Now run down the array picking a candidate prime number
     * and using it to drop out larger composites. */
    for (int p = 2; p < this.primeArray.length; p++) {
      /* If this number is really prime, then use it. */
      if (this.primeArray[p] == true) {

	/* Add it to the list of prime numbers. */
	this.cache.addPrimeToCache(new Integer(p));
	try { Thread.sleep(50); } catch (InterruptedException ie) { }

	/* Run through the numbers from p to length-1. */
	for (int n = p+1; n < this.primeArray.length; n++) {
	  /* Is this a composite of p? */
	  if (n % p == 0) {
	    this.primeArray[n] = false;
	  }
	}
      }
    }
  }

  /**
   * Tests the SieveThread class.
   *
   * @param argv The string arguments from the command line.
   */
  public static void main(String[] argv) {

    if (argv.length == 0) {
      System.out.println("Usage: java SieveThread max_prime");
      return;
    }

    PrimeCache cache = new PrimeCache();
    SieveThread sieve = new SieveThread(Integer.parseInt(argv[0]), cache);
    
    sieve.start();
    
    /* Read numbers to test from the input stream. */
    BufferedReader reader = new BufferedReader(new InputStreamReader(System.in));
    String line;
    
    try {
      while ((line = reader.readLine()) != null) {
	Integer n = new Integer(line);
	
	if (cache.isPrime(n)) {
	  System.out.println("Number ["+n.toString()+"] is prime.");
	} else {
	  System.out.println("Number ["+n.toString()+"] is composite.");
	}
      }
    } catch (IOException ioe) { }
  }
}
